import { MapStack } from '../util/stack'
import { BACK } from '../history/history-direction-name'
import { globalCache, globalStack, globalMultiKeyMap } from '../store/index'
import routerCache from '../api/router-cache'
import config from '../config/index'
import { log } from '../util/log'

function isDef(v) {
  return v !== undefined && v !== null
}

function isAsyncPlaceholder(node) {
  return node.isComment && node.asyncFactory
}

function getFirstComponentChild(children) {
  if (Array.isArray(children)) {
    for (let i = 0; i < children.length; i++) {
      const c = children[i]
      if (isDef(c) && (isDef(c.componentOptions) || isAsyncPlaceholder(c))) {
        return c
      }
    }
  }
}

function showUsingCacheKey(config, key) {
  if (config.isDebugger) {
    log(`using cache key: ${key}`)
  }
}

function showAllCacheKeys(config, globalStack) {
  if (config.isDebugger) {
    log(`all cache keys: ${JSON.stringify(globalStack.getStore())}`)
  }
}

const COMPONENT_NAME = 'router-cache'

export default {
  name: COMPONENT_NAME,
  abstract: true,
  created() {
    this.cache = Object.create(null)
    globalCache.push({
      cache: this.cache,
    })
  },
  render() {
    const slot = this.$slots.default
    const vnode = getFirstComponentChild(slot)
    const rawChild = vnode || (slot && slot[0])

    let key
    let parent = this.$parent
    let depth = 0
    let inactive = false
    while (parent && parent._routerRoot !== parent) {
      const vnodeData = parent.$vnode && parent.$vnode.data
      if (vnodeData) {
        if (vnodeData.routerView) {
          depth++
        }
        if (parent._inactive) {
          inactive = true
        }
      }
      parent = parent.$parent
    }

    const matched = this.$route.matched[depth]
    if (vnode && matched) {
      if (routerCache._isSkip) {
        routerCache._isSkip = false
        showAllCacheKeys(config, globalStack)
        return rawChild
      }

      if (config.isSingleMode) {
        key = routerCache.resolveKeyFromRoute(matched)
      } else {
        const baseKey = routerCache.resolveKeyFromRoute(matched)
        if (!globalMultiKeyMap[baseKey]) {
          globalMultiKeyMap[baseKey] = new MapStack()
        }
        const lastKey = globalMultiKeyMap[baseKey].getByIndex(0)
        if (this.$route.params[config.directionKey] !== BACK || !lastKey) {
          key = `${baseKey}_${globalMultiKeyMap[baseKey].getSize()}`
          globalMultiKeyMap[baseKey].unshift(key)
        } else {
          key = lastKey
        }
      }
      if (this.cache[key]) {
        if (inactive) {
          vnode.componentInstance = this.oldComponentInstance
        } else {
          vnode.componentInstance = this.cache[key].componentInstance
        }
        showUsingCacheKey(config, key)
      } else {
        if (!globalStack.checkFull()) {
          if (!inactive) {
            this.cache[key] = vnode
            this.oldComponentInstance = vnode.componentInstance
          }
        } else {
          const lastKey = globalStack.getFooter()
          routerCache._remove(lastKey)
          if (!inactive) {
            this.cache[key] = vnode
            this.oldComponentInstance = vnode.componentInstance
          }
        }
      }
      globalStack.unshift(key)
      vnode.data.routerCache = true
      vnode.data.keepAlive = true
    }
    showAllCacheKeys(config, globalStack)
    return rawChild
  },
  beforeDestroy() {
    for (const key in this.cache) {
      routerCache._remove(key)
    }
    let index
    for (let i = 0; i < globalCache.length; i++) {
      if (this.cache === globalCache[i]) {
        index = i
        break
      }
    }
    globalCache.splice(index, 1)
  },
}
